#!/usr/local/bin/perl5

$path = "/afs/cs.cmu.edu/user/UUUU/.blowback";
$default_mail_domain = "cs.cmu.edu";
$reallyto = $ARGV[0];
$sender = $ARGV[1];
$approved = 0;
$date = gmtime();

open LOG, ">>$path/log";
print LOG "Process $reallyto $sender\n";

# There should be no junk in these, but make sure.
$from = $sender;
$from =~ tr/-a-zA-Z0-9+@._//dc;
$from =~ tr/A-Z/a-z/;
$to = $reallyto;
$to =~ tr/-a-zA-Z0-9+@._//dc;
$to =~ tr/A-Z/a-z/;
# Where to deliver it?
$_ = $to;
if ( /^(\S+)\+\S*\@(\S+)$/ ) {
  ($mailbox, $maildomain) = /^(\S+)\+\S*\@(\S+)$/;
} else {
  if ( /^(\S+)\@(\S+)$/ ) {
    ($mailbox, $maildomain) = /^(\S+)\@(\S+)$/;
  } else {
    if ( /^\S+\+/ ) {
      ($mailbox) = /^(\S+)\+/;
      $maildomain = $default_mail_domain;
    } else {
      $mailbox = $_;
      $maildomain = $default_mail_domain;
    }
  }
}
$now_string = localtime;
print LOG "  mailbox=[$mailbox] maildomain=[$maildomain] time=[$now_string]\n";

# Read message
$message = "";
$headers = "";
$fromline = "";
while( <STDIN> ) { $headers .= $_; $fromline = $_ if(/^from:/i); last if( not /\S/ ); }
while( <STDIN> ) { $message .= $_; }

# Suspect CMU mailing lists slightly
if( $from =~ /mlist-0\.sp\.cs\.cmu\.edu$/i ) {
  $_ = $fromline;
  ($from2) = /([^ 	@\<\>\(\)\"]+@[^ 	@\<\>\(\)\"]+\.[^ 	@\<\>\(\)\"]+)/i;
  print LOG "  d-list: $from2\n";
  if( $from2 ne "" ) { $from = $from2; }
}

# Accept anything from *.edu
if( $from =~ /\.edu$/i ) {
  $approved=1;
}

# Check whitelist
if( not $approved ) {
  open WHITELIST, "$path/whitelist.txt"
    or $headers = "X-blowback-error: unable to open whitelist\n" . $headers;
  while ( <WHITELIST> ) {
    tr/-a-zA-Z0-9+@._//dc;
    tr/A-Z/a-z/;
    if ( $_ eq $from ) {
      $approved = 1;
      last;
    }
  }
  close WHITELIST;
}

# Deliver it if approved, otherwise reply.
$err=0;
if ( $approved ) {
  print LOG "  Approved\n";
  # Deliver the message
  open ( OUTPUT, "|/usr/local/bin/rdeliver $mailbox" ) or $err=1;
  print OUTPUT "X-blowback: approved $date\n";
  print OUTPUT $headers;
  print OUTPUT $message;
  close OUTPUT or $err=1;
  if($err) {
    print LOG "Error: Unable to deliver message\n";
    exit 0240;
  }
} else {
  print LOG "  Refused\n";
  # Deliver it anyway (just for debugging)
  if( 0 ) {
    open ( OUTPUT, "|/usr/local/bin/rdeliver $mailbox" ) or $err=1;
    print OUTPUT "X-blowback: rejected $date\n";
    print OUTPUT $headers;
    print OUTPUT $message;
    close OUTPUT or $err=1;
    if($err) {
      print LOG "Error: Unable to deliver message\n";
      exit 0240;
    }
  }

  # Save it in the mail queue

  # Check if a directory exists for this FROM address
  if( not -d "$path/queue/$from" ) {
    if( mkdir "$path/queue/$from", 0777 ) {}
    else {
      print LOG "Error: unable to create directory $path/queue/$from\n";
      exit 0251;
    }
  }
  # Find an unclaimed filename
  $err=0;
  opendir(DIR, "$path/queue/$from") or $err=1;
  @allfiles = readdir(DIR) or $err=1;
  closedir(DIR) or $err=1;
  $filename = 1;
  foreach (@allfiles) {
    if( /^[0-9]+$/ ) {
      if( $filename <= $_ ) {
	$filename = $_+1;
      }
    }
  }
  if($err) {
    print LOG "Error: unable to read directory $path/queue/$from\n";
    exit 0252;
  }
  # Write this message to a file
  $err=0;
  open SAVE, ">$path/queue/$from/$filename" or $err=1;
  print SAVE "X-blowback: queued $date\n" or $err=1;
  print SAVE $headers or $err=1;
  print SAVE $message or $err=1;
  close SAVE or $err=1;
  if($err) {
    print LOG "Error: unable to write file $path/queue/$from/$filename\n";
    exit 0251;
  }
  # Quit if from address is bogus
  if ( not $from =~ /@/ ) {
    print LOG "  Not sending reply because from address is clearly bogus.\n";
  } else {
    # Load the template for a bounce message
    $template = "";
    if ( open TEMPLATE, "$path/response-template.txt" ) {
      while ( <TEMPLATE> ) {
	s/\(FROM\)/$from/g;
	s/\(TO\)/$to/g;
	$template .= $_;
      }
      close TEMPLATE;
    } else {
      print LOG "Error: unable to open response template.\n";
      exit 0252;
    }
    # Send a bounce message
    $subbox = crypt($from,"qazwsx");
    use Net::SMTP;
    $smtp = Net::SMTP->new('localhost');
    $smtp->mail("");
    $smtp->to($sender);
    $smtp->data();
    $smtp->datasend("from: antispam\@$maildomain\n" .
		    "reply-to: $mailbox+approve$subbox\@$maildomain\n" .
		    "to: $from\n" .
		    "date: $date\n" .
		    "subject: please confirm your mail to $mailbox\n\n");
    $smtp->datasend($template .
		    "\n\nyou wrote:\n");
    $smtp->datasend($headers);
    $smtp->datasend($message);
    $smtp->dataend();
    $smtp->quit();
    # Done
  }
}
print LOG "  process-message.perl done\n";
close LOG;
exit 0;
