#!/usr/local/bin/perl5

$path = "/afs/cs.cmu.edu/user/UUUU/.blowback";
$default_mail_domain = "cs.cmu.edu";
$reallyto = $ARGV[0];
$sender = $ARGV[1];
$approved = 0;
$err = 0;
$date = gmtime();

open LOG, ">>$path/log";
print LOG "Temporary $reallyto $sender\n";

# There should be no junk in these, but make sure.
$from = $sender;
$from =~ tr/-a-zA-Z0-9+@._//dc;
$from =~ tr/A-Z/a-z/;
$to = $reallyto;
$to =~ tr/-a-zA-Z0-9+@._//dc;
$to =~ tr/A-Z/a-z/;
# Where to deliver it?
$_ = $to;
if ( /^(\S+)\+temp(\S+)\@(\S+)$/ ) {
  ($mailbox, $subbox, $maildomain) = /^(\S+)\+temp(\S+)\@(\S+)$/;
} else {
  if ( /^(\S+)\+temp(\S+$)/ ) {
    ($mailbox, $subbox) = /^(\S*)\+temp(\S+)$/;
    $maildomain = $default_mail_domain;
  } else {
    print LOG "Error: Invalid recipient address: $to\n";
    exit 0320;
  }
}
$now_string = localtime;
print LOG "  mailbox=[$mailbox] maildomain=[$maildomain] time=[$now_string]\n";

# Read message
$message = "";
$headers = "";
while( <STDIN> ) { $headers .= $_; last if( not /\S/ ); }
while( <STDIN> ) { $message .= $_; }

# Check temporary list
if( not $approved ) {
  open WHITELIST, "$path/temporary.txt"
    or $headers = "X-blowback-error: unable to open temporary address list\n" . $headers;
  while ( <WHITELIST> ) {
    next if /^#/;  # skip comments
    tr/-a-zA-Z0-9+@._//dc;
    tr/A-Z/a-z/;
    if ( $_ eq $subbox ) {
      $approved = 1;
      last;
    }
  }
  close WHITELIST;
}

# Deliver it if approved, otherwise reply.
$err=0;
if ( $approved ) {
  print LOG "  Approved\n";
  # Deliver the message
  open ( OUTPUT, "|/usr/local/bin/rdeliver $mailbox" ) or $err=1;
  print OUTPUT "X-blowback: approved $date\n";
  print OUTPUT $headers;
  print OUTPUT $message;
  close OUTPUT or $err=1;
  if($err) {
    print LOG "Error: Unable to deliver message\n";
    exit 0240;
  }
} else {
  print LOG "  Refused\n";

  # Save it in the mail queue

  # Check if a directory exists for this FROM address
  if( not -d "$path/queue/$from" ) {
    if( mkdir "$path/queue/$from", 0777 ) {}
    else {
      print LOG "Error: unable to create directory $path/queue/$from\n";
      exit 0251;
    }
  }
  # Find an unclaimed filename
  $err=0;
  opendir(DIR, "$path/queue/$from") or $err=1;
  @allfiles = readdir(DIR) or $err=1;
  closedir(DIR) or $err=1;
  $filename = 1;
  foreach (@allfiles) {
    if( /^[0-9]+$/ ) {
      if( $filename <= $_ ) {
	$filename = $_+1;
      }
    }
  }
  if($err) {
    print LOG "Error: unable to read directory $path/queue/$from\n";
    exit 0252;
  }
  # Write this message to a file
  $err=0;
  open SAVE, ">$path/queue/$from/$filename" or $err=1;
  print SAVE "X-blowback: queued $date\n" or $err=1;
  print SAVE $headers or $err=1;
  print SAVE $message or $err=1;
  close SAVE or $err=1;
  if($err) {
    print LOG "Error: unable to write file $path/queue/$from/$filename\n";
    exit 0251;
  }

  ## Deliver it anyway (just for debugging)
  #if( 0 ) {
  #  open ( OUTPUT, "|/usr/local/bin/rdeliver $mailbox" ) or $err=1;
  #  print OUTPUT "X-blowback: rejected $date\n";
  #  print OUTPUT $headers;
  #  print OUTPUT $message;
  #  close OUTPUT or $err=1;
  #  if($err) {
  #    print LOG "Error: Unable to deliver message\n";
  #    exit 0240;
  #  }
  #}

  # Load the template for a bounce message
  $template = "";
  if( open TEMPLATE, "$path/expired-template.txt" ) {
    while ( <TEMPLATE> ) {
      s/\(FROM\)/$from/g;
      s/\(TO\)/$to/g;
      $template .= $_;
    }
    close TEMPLATE;
  } else {
    print LOG "Error: unable to open response template.\n";
    exit 0252;
  }
  # Send a bounce message
  use Net::SMTP;
  $smtp = Net::SMTP->new('localhost');
  $smtp->mail("");
  $smtp->to($sender);
  $smtp->data();
  $smtp->datasend("from: antispam\@$maildomain\n" .
		  "to: $from\n" .
		  "date: $date\n" .
		  "subject: expired temporary address $to\n\n");
  $smtp->datasend($template .
		  "\n\nyou wrote:\n");
  $smtp->datasend($headers);
  $smtp->datasend($message);
  $smtp->dataend();
  $smtp->quit();
  # Done
}
print LOG "  process-temporary.perl done\n";
close LOG;
exit 0;
